--------------------------------------------------------------------------------
-- Title   : Reset
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Reset
--------------------------------------------------------------------------------
-- File     : reset.vhd
-- Revision : x.x.x
-- Created  : August 08, 2016
-- Updated  : November 14, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil - MAX IV Laboratory
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity reset is
  port (
    p_clkin        : in  std_logic;  -- 20MHz clock in
    p_clk500mHz    : in  std_logic;  -- 0.5Hz clock in
    p_cmd_REG0     : in  std_logic;  -- command register bit 0 ; p_CSW = "000000000000001"
    p_CSW          : in  std_logic;  -- chip select of internal write transfert
    p_PWRON_config : out std_logic; -- power on configuration signal
    p_fpga_RST     : out std_logic; -- internal reset of the FPGA (process)
    p_LED_RST      : out std_logic; -- light the red led before starting config
    p_board_RST    : out std_logic  -- mother board global reset
  );
end reset;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_reset of reset is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_cntmax : unsigned (2 downto 0) := "101"; -- 10s

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_cnt          : unsigned (2 downto 0) := (OTHERS => '0'); -- counter
  signal s_soft_reset   : std_logic := '0';
  signal s_start_config : std_logic := '0';
  signal s_config_done  : std_logic := '0';

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- FPGA process reset
  FPGA_reset : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if ((p_CSW = '1') and (p_cmd_REG0 = '1')) then
        s_soft_reset <= '1'; -- reset the FPGA's processes
      else
        s_soft_reset <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Wait 10s after power on and configure the PLL, ADC, etc.
  wait_10s : process (p_clk500mHz, s_soft_reset)
  begin
    if (s_soft_reset = '1') then
      s_cnt <= (OTHERS => '0');
      s_start_config <= '0';
    elsif (rising_edge(p_clk500mHz)) then
      if (s_cnt = c_cntmax) then
        s_cnt          <= s_cnt; -- do it one time
        s_start_config <= '1';
      else
        s_cnt          <= s_cnt + 1;
        s_start_config <= '0';
      end if;
    end if;
  end process;

  -- After 10s, send the configuration signal
  PWRON_configuration : process (p_clkin, s_soft_reset)
  begin
    if (s_soft_reset = '1') then
      s_config_done  <= '0';
      p_PWRON_config <= '0';
    elsif (rising_edge(p_clkin)) then
      if (s_start_config = '1' and s_config_done = '0') then
        p_PWRON_config <= '1';
        s_config_done  <= '1';
      else
        p_PWRON_config <= '0';
        s_config_done  <= s_config_done; -- do it one time
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_fpga_RST  <= s_soft_reset;
  p_LED_RST   <= not s_start_config;
  p_board_RST <= '0'; -- !!! SET TO 0 !!!

end rtl_reset;
