--------------------------------------------------------------------------------
-- Title   : ADC data
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Get datas from the ADC through the SPI port
--------------------------------------------------------------------------------
-- File     : ADC_data.vhd
-- Revision : x.x.x
-- Created  : August 30, 2016
-- Updated  : March 17, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ADC_lib.ADC_command_register_array;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity ADC_data is
  generic (
    g_nb_of_registers : integer;
    g_registers_size  : integer
  );
  port (
    p_clkin               : in  std_logic;
    p_reset               : in  std_logic; -- reset input
    p_ADC_command         : in  ADC_command_register_array; -- ADC convesion command
    p_ADC_datas           : out ADC_command_register_array; -- ADC convertion results
    p_ADC_datas_available : out std_logic; -- ADC data ready
    p_ADC_LE              : out std_logic; -- ADC LE
    p_ADC_CLK             : out std_logic; -- ADC clock
    p_ADC_DATA_out        : out std_logic; -- ADC data out
    p_ADC_DATA_in         : in  std_logic; -- ADC data in
    p_ADC_ENA             : out std_logic; -- ADC buffer enable A
    p_ADC_ENB             : out std_logic; -- ADC buffer enable B
    p_ADC_ENC             : out std_logic; -- ADC buffer enable C
    p_ADC_END             : out std_logic  -- ADC buffer enable D
  );
end entity ADC_data;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_ADC_data of ADC_data is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_start_data      : std_logic := '0';
  signal s_finished_data   : std_logic := '0';
  signal s_registers_index : integer   := 0;
  signal s_SPI_cnt         : unsigned (7 downto 0) := (others => '0'); -- state machine counter
  signal s_data_out        : std_logic_vector (g_registers_size downto 0) := (others => '0');
  signal s_data_in         : std_logic_vector (g_registers_size downto 0) := (others => '0');
  signal s_push_data_out   : std_logic := '0';

  -- ADC chip signals
  signal s_ADC_LE       : std_logic := '1';
  signal s_ADC_CLK      : std_logic := '0';
  signal s_ADC_DATA_out : std_logic := '0';
  signal s_ADC_ENA      : std_logic := '1';
  signal s_ADC_ENB      : std_logic := '0';
  signal s_ADC_ENC      : std_logic := '0';
  signal s_ADC_END      : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Start the data of the ADC
  -- The data start by seting the p_reset
  -- It stops when all registers has been pushed out
  -- Buffers between the digital board and the RF board are set and unset
  -- during the transmission
  start_data : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if (p_reset = '1') then
        -- Start the process of pushing the registers to the output
        s_start_data <= '1';
        -- Set the buffer signals
        s_ADC_ENA <= '1';
        s_ADC_ENB <= '1';
        s_ADC_ENC <= '1';
        s_ADC_END <= '1';
      elsif (s_start_data = '1') then
        if (s_finished_data = '1') then
          -- All registers has been pushed out
          -- Stop and wait for next commande through p_reset signal
          s_start_data <= '0'; -- stay here forever until a reset
          -- Unset the buffer signals
          s_ADC_ENA <= '1';
          s_ADC_ENB <= '0';
          s_ADC_ENC <= '0';
          s_ADC_END <= '0';
        end if;
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- SPI state machine
  -- Push each bit of a register to the output then fetch the next register
  -- The current register is selected with s_registers_index
  SPI_manager : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if (s_start_data = '1') then
        case s_SPI_cnt is
          when to_unsigned(0, s_SPI_cnt'HIGH + 1) =>
            -- Start to push each bit of the current
            -- register to the output
            s_ADC_LE  <= '0';
            s_SPI_cnt <= s_SPI_cnt + 1;
            s_push_data_out <= '1';
          when to_unsigned((g_registers_size * 2) + 2, s_SPI_cnt'HIGH + 1) =>
            -- Stop to push each bit out
            s_SPI_cnt       <= s_SPI_cnt + 1;
            s_push_data_out <= '0';
          when to_unsigned((g_registers_size * 2) + 3, s_SPI_cnt'HIGH + 1) =>
            -- Latch data to ADC
            s_SPI_cnt <= s_SPI_cnt + 1;
            s_ADC_LE  <= '1';
          when to_unsigned((g_registers_size * 2) + 4, s_SPI_cnt'HIGH + 1) =>
            -- End of transmission of the current register
            s_SPI_cnt       <= (others => '0');
            s_push_data_out <= '0';
            if (s_registers_index = g_nb_of_registers) then
              -- All registers had been pushed out
              s_finished_data <= '1';
            else
              -- Switch to the next register
              s_registers_index <= s_registers_index + 1;
            end if;
          when others =>
            s_SPI_cnt <= s_SPI_cnt + 1;
        end case;
      else
        s_SPI_cnt       <= (others => '0');
        s_push_data_out <= '0';
        -- Initialization of the index to the first register
        s_registers_index <= 0;
        s_finished_data   <= '0';
        s_ADC_LE <= '1';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Shift register to push each bit of the current register to the output
  bit_transmission : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if (s_push_data_out = '1') then
        case s_SPI_cnt(0) is
          when '1' =>
            -- Push out one bit
            s_ADC_DATA_out <= s_data_out(g_registers_size);
            s_data_out(g_registers_size downto 1) <= s_data_out((g_registers_size - 1) downto 0);
            -- Get one bit in
            s_data_in <= s_data_in((g_registers_size - 1) downto 0) & p_ADC_DATA_in;
            -- Generate clock
            s_ADC_CLK <= '1';
          when others =>
            s_ADC_CLK <= '0';
        end case;
      else
        -- Idle state
        s_ADC_DATA_out <= '0';
        s_data_out     <= p_ADC_command(s_registers_index);
        p_ADC_datas(s_registers_index) <= s_data_in;
        s_ADC_CLK      <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_ADC_LE              <= s_ADC_LE;        -- ADC LE
  p_ADC_CLK             <= s_ADC_CLK;       -- ADC clock
  p_ADC_DATA_out        <= s_ADC_DATA_out;  -- ADC data out
  p_ADC_ENA             <= s_ADC_ENA;       -- ADC buffer enable A
  p_ADC_ENB             <= s_ADC_ENB;       -- ADC buffer enable B
  p_ADC_ENC             <= s_ADC_ENC;       -- ADC buffer enable C
  p_ADC_END             <= s_ADC_END;       -- ADC buffer enable D
  p_ADC_datas_available <= s_finished_data; -- datas retrived

end architecture rtl_ADC_data;
