--------------------------------------------------------------------------------
-- Title   : ADC
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Analog to Digital driver
--               Redback the skew between the input and output of the IQ modulator
--               RF signals
--------------------------------------------------------------------------------
-- File     : ADC_top.vhd
-- Revision : x.x.x
-- Created  : August 8, 2016
-- Updated  : November 13, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ADC_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity ADC_top is
  port (
    p_reset      : in  std_logic; -- reset input
    p_clkin      : in  std_logic; -- CLK input
    p_get_data   : in  std_logic_vector (1 downto 0);  -- start retrieve data
    p_ADC_datas  : out std_logic_vector (33 downto 0); -- ADC convertion results
    p_data_ready : out std_logic; -- data from ADC heve been retrieved
    p_ADC_RSTn   : out std_logic; -- ADC reset
    p_ADC_CSn    : out std_logic; -- ADC chip select
    p_ADC_SCK    : out std_logic; -- ADC clock
    p_ADC_MOSI   : out std_logic; -- ADC Master Out Slave In
    p_ADC_MISO   : in  std_logic; -- ADC Master In Slave Out
    p_ADC_ALARM  : in  std_logic; -- ADC alarm
    p_ADC_ENA    : out std_logic; -- ADC buffer enable A
    p_ADC_ENB    : out std_logic; -- ADC buffer enable B
    p_ADC_ENC    : out std_logic; -- ADC buffer enable C (noted as ADC_ENC in schematics)
    p_ADC_END    : out std_logic  -- ADC buffer enable D (noted as ADC_END in schematics)

--    p_test       : out   std_logic_vector (4 downto 0)

  );
end entity ADC_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_ADC_top of ADC_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_ADC_LE_config        : std_logic := '0';
  signal s_ADC_CLK_config       : std_logic := '0';
  signal s_ADC_MOSI_config      : std_logic := '0';
  signal s_ADC_ENA_config       : std_logic := '0';
  signal s_ADC_ENB_config       : std_logic := '0';
  signal s_ADC_ENC_config       : std_logic := '0';
  signal s_ADC_END_config       : std_logic := '0';
  signal s_ADC_datas_available  : std_logic := '0';
  signal s_ADC_LE_data          : std_logic := '0';
  signal s_ADC_CLK_data         : std_logic := '0';
  signal s_ADC_MOSI_data        : std_logic := '0';
  signal s_ADC_ENA_data         : std_logic := '0';
  signal s_ADC_ENB_data         : std_logic := '0';
  signal s_ADC_ENC_data         : std_logic := '0';
  signal s_ADC_END_data         : std_logic := '0';
  signal s_ADC_datas            : ADC_command_register_array;
  signal s_ADC_command          : ADC_command_register_array;
  signal s_ADC_start_convertion : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Configuration of the ADC trigerred by the p_reset signal
  configure_ADC : entity work.ADC_configuration (rtl_ADC_configuration)
    generic map (
      g_registers       => c_ADC_reg_configuration,
      g_nb_of_registers => c_ADC_reg_configuration'HIGH,
      g_registers_size  => c_ADC_reg_configuration(0)'HIGH )
    port map (
      p_clkin    => p_clkin,
      p_reset    => p_reset,
      p_ADC_LE   => s_ADC_LE_config,
      p_ADC_CLK  => s_ADC_CLK_config,
      p_ADC_DATA => s_ADC_MOSI_config,
      p_ADC_ENA  => s_ADC_ENA_config,
      p_ADC_ENB  => s_ADC_ENB_config,
      p_ADC_ENC  => s_ADC_ENC_config,
      p_ADC_END  => s_ADC_END_config
    );

  ------------------------------------------------------------------------------
  -- Retrieve datas from the ADC trigerred by the p_get_data signal
  data_ADC : entity work.ADC_data (rtl_ADC_data)
    generic map (
      g_nb_of_registers => c_ADC_reg_CH0'HIGH,
      g_registers_size  => c_ADC_reg_CH0(0)'HIGH )
    port map (
      p_clkin               => p_clkin,
      p_reset               => s_ADC_start_convertion,
      p_ADC_command         => s_ADC_command,
      p_ADC_datas           => s_ADC_datas,
      p_ADC_datas_available => s_ADC_datas_available,
      p_ADC_LE              => s_ADC_LE_data,
      p_ADC_CLK             => s_ADC_CLK_data,
      p_ADC_DATA_out        => s_ADC_MOSI_data,
      p_ADC_DATA_in         => p_ADC_MISO,
      p_ADC_ENA             => s_ADC_ENA_data,
      p_ADC_ENB             => s_ADC_ENB_data,
      p_ADC_ENC             => s_ADC_ENC_data,
      p_ADC_END             => s_ADC_END_data
    );

  ------------------------------------------------------------------------------
  -- DAC channel selection
  process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      case p_get_data is
        when "01" =>
          -- ADC CH0
          s_ADC_command <= c_ADC_reg_CH0;
          s_ADC_start_convertion <= '1';
        when "10" =>
          -- ADC CH1
          s_ADC_command <= c_ADC_reg_CH1;
          s_ADC_start_convertion <= '1';
        when others =>
          s_ADC_command <= s_ADC_command;
          s_ADC_start_convertion <= '0';
      end case;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_ADC_CSn    <= s_ADC_LE_config   and s_ADC_LE_data;
  p_ADC_SCK    <= s_ADC_CLK_config  or  s_ADC_CLK_data;
  p_ADC_MOSI   <= s_ADC_MOSI_config or  s_ADC_MOSI_data;
  p_ADC_ENA    <= s_ADC_ENA_config  or  s_ADC_ENA_data;
  p_ADC_ENB    <= s_ADC_ENB_config  or  s_ADC_ENB_data;
  p_ADC_ENC    <= s_ADC_ENC_config  or  s_ADC_ENC_data;
  p_ADC_END    <= s_ADC_END_config  or  s_ADC_END_data;
  p_data_ready <= s_ADC_datas_available;

  p_ADC_datas <= s_ADC_datas(1);
  p_ADC_RSTn  <= '1'; -- ADC reset

--  p_test(0) <= s_ADC_CLK_data;
--  p_test(1) <= s_ADC_MOSI_data;
--  p_test(2) <= '0';
--  p_test(3) <= '0';
--  p_test(4) <= p_ADC_MISO;

end architecture rtl_ADC_top;
