--------------------------------------------------------------------------------
-- Title   : BBB I2C access
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : I2C access with the BBB
--               Slave mode ; 7 bits address ; 2 back-to-back 8 bits = 16 bits
--               data
--------------------------------------------------------------------------------
-- File     : BBB_I2C.vhd
-- Revision : x.x.x
-- Created  : September 12, 2016
-- Updated  : December 11, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.BBB_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity BBB_I2C is
  port (
    p_clkin             : in    std_logic; -- FPGA clock
    p_reset             : in    std_logic; -- reset input
    p_address           : out   std_logic_vector (6 downto 0);
    p_read_register     : out   std_logic_vector (23 downto 0);
    p_read_strobe       : out   std_logic; -- data from master available
    p_write_register    : in    std_logic_vector (7 downto 0);
    p_write_request     : out   std_logic; -- ask data to be send to master
    p_write_data_ready  : in    std_logic; -- data to be send to master ready
    p_SCL               : in    std_logic; -- I2C SCL
    p_SDA               : inout std_logic; -- I2C SDA

    p_test              : out   std_logic_vector (4 downto 0)
  );
end entity BBB_I2C;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_BBB_I2C of BBB_I2C is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_SCL_cnt_max : unsigned (7 downto 0) := "11111111"; -- 255/20MHz=12.75s

  type header_cycles is (hd_idle,              -- waitning
                         hd_decode_address,    -- decode the address
                         hd_decode_cmd,        -- decode command
                         hd_acknowledge_start, -- push acknowledge bit out
                         hd_acknowledge_stop); -- release acknowledge bit

  type read_cycles is (rd_idle,               -- waitning
                       rd_first_byte,         -- read first byte from master
                       rd_acknowlege1_start,  -- push acknowledge bit out
                       rd_acknowledge1_stop,  -- release acknowledge bit
                       rd_second_byte,        -- read second byte from master
                       rd_acknowledge2_start, -- push acknowledge bit out
                       rd_acknowledge2_stop,  -- release acknowledge bit
                       rd_third_byte,         -- read third byte from master
                       rd_acknowledge3_start, -- push acknowledge bit out
                       rd_acknowledge3,       -- set data ready strobe
                       rd_acknowledge4_stop); -- release acknowledge bit

  type write_cycles is (wr_idle,               -- waitning
                        wr_set_data_request,   -- request data
                        wr_wait_data,          -- data available
                        wr_first_byte,         -- write first byte to master
                        wr_wait_state,         -- wait state
                        wr_acknowledge2_start, -- get acknowledge bit
                        wr_acknowledge2_stop); -- check acknowledge bit

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_SDA             : std_logic := '0';
  signal s_SCL             : std_logic := '0';
  signal s_hd_state        : header_cycles := hd_idle;
  signal s_rd_state        : read_cycles   := rd_idle;
  signal s_wr_state        : write_cycles  := wr_idle;
  signal s_SCL_cnt1        : unsigned (7 downto 0) := (others => '0');
  signal s_SCL_cnt2        : unsigned (7 downto 0) := (others => '0');
  signal s_start_condition : std_logic  := '0';
  signal s_stop_condition  : std_logic  := '0';
  signal s_SDA_previous1   : std_logic  := '0';
  signal s_SDA_previous2   : std_logic  := '0';
  signal s_check_SDA       : std_logic  := '0';
  signal s_check_SCL       : std_logic  := '0';
  signal s_SDA_rising      : std_logic  := '0';
  signal s_SDA_falling     : std_logic  := '0';
  signal s_SCL_rising      : std_logic  := '0';
  signal s_SCL_fl          : std_logic  := '0';
  signal s_SCL_falling     : std_logic  := '0';
  signal s_cmd             : std_logic  := '0';
  signal s_hd_direction    : std_logic  := '0';
  signal s_hd_out          : std_logic  := '1';
  signal s_rd_direction    : std_logic  := '0';
  signal s_rd_out          : std_logic  := '1';
  signal s_wr_direction    : std_logic  := '0';
  signal s_wr_out          : std_logic  := '1';
  signal s_wr_ack          : std_logic  := '1';
  signal s_direction       : std_logic  := '0';
  signal s_out             : std_logic  := '0';
  signal s_start_read      : std_logic  := '0';
  signal s_start_write     : std_logic  := '0';
  signal s_address         : std_logic_vector (6 downto 0) := (others => '0');
  signal s_hd_counter      : unsigned (2 downto 0) := (others => '0');
  signal s_data_in         : std_logic_vector (23 downto 0) := (others => '0');
  signal s_rd_counter      : unsigned (4 downto 0) := (others => '0');
  signal s_data_out        : std_logic_vector (7 downto 0) := (others => '0');
  signal s_wr_counter      : unsigned (3 downto 0) := (others => '0');

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Debounce the inputs
  debounce_SDA : entity work.BBB_debounce (rtl_BBB_debounce)
    generic map (
      g_size => 5 )
    port map (
      p_clkin        => p_clkin,
      p_in           => p_SDA,
      p_rising_edge  => s_SDA_rising,
      p_falling_edge => s_SDA_falling,
      p_out          => s_SDA
    );

  debounce_SCL : entity work.BBB_debounce (rtl_BBB_debounce)
    generic map (
      g_size => 5 )
    port map (
      p_clkin        => p_clkin,
      p_in           => p_SCL,
      p_rising_edge  => s_SCL_rising,
      p_falling_edge => s_SCL_fl,
      p_out          => s_SCL
    );

  wait_state_SCL : entity work.BBB_wait_state (rtl_BBB_wait_state)
    generic map (
      g_size  => 3,
      g_delay => "1000" -- delay the clock by 400 ns
    )
    port map (
      p_clkin => p_clkin,
      p_in    => s_SCL_fl,
      p_out   => s_SCL_falling
    );

  ------------------------------------------------------------------------------
  -- Start condition on I2C bus => falling edge of SDA while SCL is set
  -- SCL _____________
  --                  \__________________________
  --     _______
  -- SDA        \________________________________
  --
  start_detection_SCL : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if ((s_start_condition = '0') and (s_direction = '0') and (s_SCL = '1')) then
        -- SCL must stay at 1 for more than a half a period
        if (s_SCL_cnt1 = c_SCL_cnt_max) then
          s_check_SDA <= '1';       -- check start condition
          s_SCL_cnt1  <= s_SCL_cnt1; -- stay in this state until a start is detected
        else
          s_check_SDA <= '0';
          s_SCL_cnt1  <= s_SCL_cnt1 + 1;
        end if;
      else
        s_check_SDA <= '0';
        s_SCL_cnt1  <= (others => '0');
      end if;
    end if;
  end process;

  start_detection_SDA : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      s_SDA_previous1 <= s_SDA;
      if (s_check_SDA = '1') then
        -- Check if there is a 1 to 0 transition on SDA while SCL is 1
        if ((s_SDA_previous1 = '1') and (s_SDA ='0')) then
          s_start_condition <= '1';
        else
          s_start_condition <= '0';
        end if;
      else
        s_start_condition <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Stop condition on I2C bus => rising edge of SDA while SCL is set
  --          ___________________________________
  -- SCL ____/
  --               ______________________________
  -- SDA _________/
  --
  stop_detection_SDA : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      s_SDA_previous2 <= s_SDA;
      if (s_stop_condition = '0' and (s_direction = '0')) then
        -- Check if there is a 0 to 1 transition on SDA
        if ((s_SDA_previous2 = '0') and (s_SDA ='1')) then
          s_check_SCL <= '1';
        else
          s_check_SCL <= s_check_SCL;
        end if;
      else
        s_check_SCL <= '0';
      end if;
    end if;
  end process;

  stop_detection_SCL : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if ((s_check_SCL = '1') and (s_SCL = '1')) then
        -- SCL must stay at 1 for more than a half a period
        if (s_SCL_cnt2 = c_SCL_cnt_max) then
          s_stop_condition <= '1';
          s_SCL_cnt2       <= s_SCL_cnt2;
        else
          s_stop_condition <= '0';
          s_SCL_cnt2       <= s_SCL_cnt2 + 1;
        end if;
      else
        s_stop_condition <= '0';
        s_SCL_cnt2       <= (others => '0');
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- I2C address and command decode
  -- 7 bits address ; 1 bit command
  I2C_address_cmd_decode : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      ----------------------------
      -- Start condition
      if (s_start_condition = '1') then
        s_hd_state <= hd_decode_address;
      end if;
      ----------------------------
      -- Stop condition
      if (s_stop_condition = '1') then
        s_hd_state <= hd_idle;
      end if;
      ----------------------------
      -- Sate Machine
      case s_hd_state is
        ----------------------------
        -- Idle state
        when hd_idle =>
          -- Wait for a start condition
          s_hd_direction <= '0'; -- in direction
          s_hd_out       <= '1';
          s_start_read   <= '0';
          s_start_write  <= '0';
          s_hd_counter   <= (others => '0');
        ----------------------------
        -- Address decode
        when hd_decode_address =>
          -- Decode the I2C address
          if (s_SCL_rising = '1') then
            s_hd_counter <= s_hd_counter + 1;
            s_address    <= s_address(5 downto 0) & s_SDA;
            if (s_hd_counter = 6) then
              s_hd_state <= hd_decode_cmd;
            end if;
          end if;
        ----------------------------
        -- Command decode
        when hd_decode_cmd =>
          -- Does the address match ?
          if (s_address(6 downto 4) = c_BBB_base_address(6 downto 4)) then
            p_address <= s_address;
            -- Decode the command bit
            if (s_SCL_rising = '1') then
              s_cmd      <= s_SDA; -- Read / Write
              s_hd_state <= hd_acknowledge_start;
            end if;
          else
            -- Address dosn't match, go back to wait state
            s_hd_state <= hd_idle;
          end if;
        ----------------------------
        -- Acknowledge
        when hd_acknowledge_start =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_hd_direction <= '1'; -- output direction
            s_hd_out <= '0';
            if (s_cmd = '1') then
              -- Master read ; FPGA write
              s_start_write <= '1';
            else
              -- Master write ; FPGA read
              s_start_write <= '0';
            end if;
            s_hd_state <= hd_acknowledge_stop;
          end if;
        when hd_acknowledge_stop =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            if (s_cmd = '0') then
              -- Master read ; FPGA write
              s_start_read  <= '1';
            else
              -- Master write ; FPGA read
              s_start_read  <= '0';
            end if;
            s_hd_state <= hd_idle;
          end if;
        ----------------------------
        --
        when others =>
          s_hd_state <= hd_idle;
      end case;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- I2C write ; read from the master
  -- 24 bits data
  I2C_read_from_master : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      ----------------------------
      -- Start condition
      if (s_start_condition = '1') then
        s_rd_state <= rd_idle;
      end if;
      ----------------------------
      -- Stop condition
      if (s_stop_condition = '1') then
        s_rd_state <= rd_idle;
      end if;
      ----------------------------
      -- Sate Machine
      case s_rd_state is
        ----------------------------
        -- Idle state
        when rd_idle =>
          -- Wait for a start condition
          s_rd_direction <= '0'; -- in direction
          s_rd_out       <= '1';
          s_data_in      <= (others => '0');
          s_rd_counter   <= (others => '0');
          p_read_strobe  <= '0';
          if (s_start_read = '1') then
            s_rd_state <= rd_first_byte;
          end if;
        ----------------------------
        -- Read from SDA
        when rd_first_byte =>
          if (s_SCL_rising = '1') then
            s_rd_counter <= s_rd_counter + 1;
            s_data_in    <= s_data_in(22 downto 0) & s_SDA;
            if (s_rd_counter = 7) then
              s_rd_state <= rd_acknowlege1_start;
            end if;
          end if;
        ----------------------------
        -- Acknowledge
        when rd_acknowlege1_start =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '1'; -- output direction
            s_rd_out   <= '0';
            s_rd_state <= rd_acknowledge1_stop;
          end if;
        when rd_acknowledge1_stop =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '0'; -- in direction
            s_rd_out       <= '1';
            s_rd_state     <= rd_second_byte;
          end if;
        ----------------------------
        -- Read from SDA
        when rd_second_byte =>
          if (s_SCL_rising = '1') then
            s_rd_counter <= s_rd_counter + 1;
            s_data_in    <= s_data_in(22 downto 0) & s_SDA;
            if (s_rd_counter = 15) then
              s_rd_state <= rd_acknowledge2_start;
            end if;
          end if;
        ----------------------------
        -- Acknowledge
        when rd_acknowledge2_start =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '1'; -- output direction
            s_rd_out   <= '0';
            s_rd_state <= rd_acknowledge2_stop;
          end if;
        when rd_acknowledge2_stop =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '0'; -- in direction
            s_rd_out       <= '1';
            s_rd_state     <= rd_third_byte;
          end if;
        ----------------------------
        -- Read from SDA
        when rd_third_byte =>
          if (s_SCL_rising = '1') then
            s_rd_counter <= s_rd_counter + 1;
            s_data_in    <= s_data_in(22 downto 0) & s_SDA;
            if (s_rd_counter = 23) then
              s_rd_state <= rd_acknowledge3_start;
            end if;
          end if;
        ----------------------------
        -- Acknowledge
        when rd_acknowledge3_start =>
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '1'; -- output direction
            s_rd_out   <= '0';
            s_rd_state <= rd_acknowledge3;
          end if;
        when rd_acknowledge3 =>
          -- Set the data ready strobe for one period
          p_read_register <= s_data_in;
          p_read_strobe   <= '1';
          s_rd_state      <= rd_acknowledge4_stop;
        when rd_acknowledge4_stop =>
          p_read_strobe   <= '0';
          -- Wait for the next falling edge of SCL
          if (s_SCL_falling = '1') then
            s_rd_direction <= '0'; -- in direction
            s_rd_out       <= '1';
            s_rd_state <= rd_idle;
          end if;
        ----------------------------
        --
        when others =>
          -- Wait for a transmission on the I2C bus
          s_rd_state <= rd_idle;
      end case;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- I2C read ; write to master
  -- 8 bits data
  I2C_write_to_master : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      ----------------------------
      -- Start condition
      if (s_start_condition = '1') then
        s_wr_state <= wr_idle;
      end if;
      ----------------------------
      -- Stop condition
      if (s_stop_condition = '1') then
        s_wr_state <= wr_idle;
      end if;
      ----------------------------
      -- Sate Machine
      case s_wr_state is
        ----------------------------
        -- Idle state
        when wr_idle =>
          -- Wait for a start condition
          s_wr_direction  <= '0'; -- in direction
          s_wr_ack        <= '1';
          s_wr_out        <= '1';
          s_data_out      <= (others => '0');
          s_wr_counter    <= (others => '0');
          p_write_request <= '0';
          if (s_start_write = '1') then
            s_wr_state <= wr_set_data_request;
          end if;
        ----------------------------
        -- Send data request
        when wr_set_data_request =>
          p_write_request <= '1';
          s_wr_state <= wr_wait_data;
        ----------------------------
        -- Wait data to be available
        when wr_wait_data =>
          p_write_request <= '0';
          if (p_write_data_ready = '1') then
            s_data_out <= p_write_register;
            s_wr_out   <= p_write_register(7);
            s_wr_state <= wr_first_byte;
          end if;
        ----------------------------
        -- Write to SDA
        when wr_first_byte =>
          s_wr_direction <= '1'; -- out direction
          if (s_SCL_falling = '1') then
            s_wr_out     <= s_data_out(7);
            s_wr_counter <= s_wr_counter + 1;
            s_data_out(7 downto 1) <= s_data_out(6 downto 0);
            if (s_wr_counter = 7) then
              s_wr_state <= wr_wait_state;
            end if;
          end if;
        ----------------------------
        -- Wait state
        when wr_wait_state =>
          if (s_SCL_falling = '1') then
            s_wr_state <= wr_acknowledge2_start;
          end if;
        ----------------------------
        -- Acknowledge
        when wr_acknowledge2_start =>
          s_wr_direction <= '0'; -- in direction
          -- Get acknowledge from master
          if (s_SCL_rising = '1') then
            s_wr_ack   <= s_SDA;
            s_wr_state <= wr_acknowledge2_stop;
          end if;
        when wr_acknowledge2_stop =>
          -- Check acknowledge
          -- Go back to idle anyway, write is finished
          if (s_SCL_falling = '1') then
            s_wr_state <= wr_idle;
          end if;
        ----------------------------
        --
        when others =>
          -- Wait for a transmission on the I2C bus
          s_wr_state <= wr_idle;
      end case;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_out <= s_hd_out and s_rd_out and s_wr_out;
  s_direction <= s_hd_direction or s_rd_direction or s_wr_direction;
  p_SDA <= s_out when s_direction = '1' else 'Z'; -- BBB data

  p_test(0) <= s_SCL;
  p_test(1) <= s_wr_out;
  p_test(2) <= s_start_read;
  p_test(3) <= s_direction;
  p_test(4) <= s_out;

end architecture rtl_BBB_I2C;
