--------------------------------------------------------------------------------
-- Title   : BBB debounce
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : debounce the input signal
--------------------------------------------------------------------------------
-- File     : BBB_debounce.vhd
-- Revision : x.x.x
-- Created  : September 12, 2016
-- Updated  : November 02, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity BBB_debounce is
  generic (
    g_size : integer
  );
  port (
    p_clkin        : in  std_logic; -- clock
    p_in           : in  std_logic; -- input signal
    p_rising_edge  : out std_logic; -- output rising edge detector
    p_falling_edge : out std_logic; -- output falling edge detector
    p_out          : out std_logic  -- output debouced signal
  );
end BBB_debounce;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_BBB_debounce of BBB_debounce is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_buffer  : std_logic_vector (g_size downto 0) := (others => '0');
  signal s_rising  : std_logic := '0';
  signal s_falling : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Components
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Synchronization of the external async signal with the clock
  process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      s_buffer(0) <= p_in;
      for I in 1 to g_size loop
        s_buffer(I) <= s_buffer(I - 1);
      end loop;
      s_rising  <= s_buffer(g_size - 1) and (not s_buffer(g_size));
      s_falling <= s_buffer(g_size) and (not s_buffer(g_size - 1));
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_rising_edge  <= s_rising;
  p_falling_edge <= s_falling;
  p_out          <= s_buffer(g_size);

end rtl_BBB_debounce;
