--------------------------------------------------------------------------------
-- Title   : BBB wait state
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : add a wait state to the input signal.
--               The wait state pulse is one CLK width.
--               The input signal is delyed by g_delay
--------------------------------------------------------------------------------
-- File     : BBB_wait_state.vhd
-- Revision : x.x.x
-- Created  : December 11, 2017
-- Updated  : December 11, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity BBB_wait_state is
  generic (
    g_size  : integer;
    g_delay : unsigned
  );
  port (
    p_clkin : in  std_logic; -- clock
    p_in    : in  std_logic; -- input signal
    p_out   : out std_logic  -- delayed signal
  );
end entity BBB_wait_state;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_BBB_wait_state of BBB_wait_state is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_stop  : std_logic := '0';
  signal s_start : std_logic := '0';
  signal s_cnt   : unsigned (g_size downto 0) := (others => '0'); -- state machine 
  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Detect a start condition on p_in (it goes high for one period)
  start_detection : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if (p_in = '1') then
        s_start <= '1';
      elsif (s_stop = '1') then
        s_start <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Delay the input signal by the requested number of clock time
  wait_state : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if (s_start = '1') then
        if (s_cnt = g_delay) then
          s_cnt  <= (others => '0');
          s_stop <= '1';
        else
          s_cnt  <= s_cnt + 1;
          s_stop <= '0';
        end if;
      else
        s_cnt  <= (others => '0');
        s_stop <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_out <= s_stop;

end architecture rtl_BBB_wait_state;
