--------------------------------------------------------------------------------
-- Title   : clock top
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : sub frequecy clocks generated from the main clock
--------------------------------------------------------------------------------
-- File     : clk_top.vhd
-- Revision : x.x.x
-- Created  : August 2, 2016
-- Updated  : August 22, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2016 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.clk_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clk_top is
  port (
    p_clk100MHz  : in  std_logic; -- input clock 100 MHz
    p_clk20MHz   : out std_logic; -- output clock 20 MHz
    p_reset      : in  std_logic; -- reset input
    p_clk500mHz  : out std_logic  -- output clock 0.5 Hz
  );
end entity clk_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clk_top of clk_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_cntmax : unsigned (2 downto 0) := "100"; -- /5
  -- dividion by 20^7 : (24, 21) : X^24 + X^21
  constant c_lfsr_1Hz : lfsr_polynome (0 to 1) := (24, 21);

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_clk20MHz  : std_logic := '0';
  signal s_clk1Hz    : std_logic := '0';
  signal s_clk500mHz : std_logic := '0';
  signal s_cnt       : unsigned (2 downto 0) := (OTHERS => '0'); -- counter

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------
  attribute PERIOD : string;
  attribute PERIOD of p_clk20MHz : signal is "50 ns";

  attribute BUFFER_TYPE : string;
  attribute BUFFER_TYPE of p_clk20MHz : signal is "BUFG";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- 20 MHz clock
  -- ADC, DAC, and PLL serial clock. ADC clock is limited to 18 MHz max
  clk_20MHz : process (p_clk100MHz)
  begin
    if (rising_edge(p_clk100MHz)) then
      if (s_cnt = c_cntmax) then
        s_cnt      <= (OTHERS => '0');
        s_clk20MHz <= '1';
      else
        s_cnt      <= s_cnt + 1;
        s_clk20MHz <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- 1 Hz clock
  -- Slow processes
  lfsr_1Hz : entity work.clk_lsfr (rtl_clk_lsfr)
    generic map (
      g_lfsr => integer (1633889), -- polynome equality value
      g_polynome => c_lfsr_1Hz )   -- polynome powers
    port map (
      p_clkin  => s_clk20MHz,
      p_clkout => s_clk1Hz
    );

  ------------------------------------------------------------------------------
  -- 0.5 Hz clock square
  -- LEDs
  process (s_clk1Hz)
  begin
    if (rising_edge(s_clk1Hz)) then
      s_clk500mHz <= not s_clk500mHz;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_clk20MHz  <= s_clk20MHz;
  p_clk500mHz <= s_clk500mHz;

end architecture rtl_clk_top;
