--------------------------------------------------------------------------------
-- Title   : DAC
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Digital to Analog
--               Provide the I and the Q voltage of the IQ modulator
--               Q = DAC A ; I = DAC B
--------------------------------------------------------------------------------
-- File     : DAC_top.vhd
-- Revision : x.x.x
-- Created  : August 22, 2016
-- Updated  : November 10, 2016
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.DAC_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity DAC_top is
  port (
    p_reset        : in  std_logic; -- reset input
    p_clkin        : in  std_logic; -- CLK input
    p_DAC_reg      : in  std_logic_vector (31 downto 0);  -- I and Q setting
    p_write_strobe : in  std_logic; -- write to DAC request
    p_DAC_CSn      : out std_logic; -- DAC chip select
    p_DAC_SCK      : out std_logic; -- DAC clock
    p_DAC_MOSI     : out std_logic; -- DAC Master Out Slave In
    p_DAC_MISO     : in  std_logic; -- DAC Master In Slave Out
    p_DAC_ENA      : out std_logic; -- DAC buffer enable A
    p_DAC_ENB      : out std_logic  -- DAC buffer enable B
  );
end entity DAC_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_DAC_top of DAC_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_DAC_CSn_cfg  : std_logic := '0';
  signal s_DAC_SCK_cfg  : std_logic := '0';
  signal s_DAC_MOSI_cfg : std_logic := '0';
  signal s_DAC_ENA_cfg  : std_logic := '0';
  signal s_DAC_ENB_cfg  : std_logic := '0';
  signal s_DAC_CSn_set  : std_logic := '0';
  signal s_DAC_SCK_set  : std_logic := '0';
  signal s_DAC_MOSI_set : std_logic := '0';
  signal s_DAC_ENA_set  : std_logic := '0';
  signal s_DAC_ENB_set  : std_logic := '0';

  signal s_write_strobe : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Configuration of the DAC trigerred by the p_reset signal at power-up
  configure_DAC : entity work.DAC_configuration (rtl_DAC_configuration)
    generic map (
      g_registers       => c_DAC_reg_configuration,
      g_nb_of_registers => c_DAC_reg_configuration'HIGH,
      g_registers_size  => c_DAC_reg_configuration(0)'HIGH )
    port map (
      p_clkin    => p_clkin,
      p_reset    => p_reset,
      p_DAC_LE   => s_DAC_CSn_cfg,
      p_DAC_CLK  => s_DAC_SCK_cfg,
      p_DAC_DATA => s_DAC_MOSI_cfg,
      p_DAC_ENA  => s_DAC_ENA_cfg,
      p_DAC_ENB  => s_DAC_ENB_cfg
    );

  ------------------------------------------------------------------------------
  -- Set I or Q
  -- Selection of I or Q is made in Hermes module
  set_DAC : entity work.DAC_setIQ (rtl_DAC_setIQ)
    port map (
      p_clkin    => p_clkin,
      p_reset    => s_write_strobe,
      p_DAC_reg  => p_DAC_reg,
      p_DAC_LE   => s_DAC_CSn_set,
      p_DAC_CLK  => s_DAC_SCK_set,
      p_DAC_DATA => s_DAC_MOSI_set,
      p_DAC_ENA  => s_DAC_ENA_set,
      p_DAC_ENB  => s_DAC_ENB_set
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_DAC_CSn  <= s_DAC_CSn_cfg  and s_DAC_CSn_set;
  p_DAC_SCK  <= s_DAC_SCK_cfg  or  s_DAC_SCK_set;
  p_DAC_MOSI <= s_DAC_MOSI_cfg or  s_DAC_MOSI_set;
  p_DAC_ENA  <= s_DAC_ENA_cfg  or  s_DAC_ENA_set;
  p_DAC_ENB  <= s_DAC_ENB_cfg  or  s_DAC_ENB_set;

  s_write_strobe <= p_write_strobe and (not p_reset); -- priority to the reset

end architecture rtl_DAC_top;
