--------------------------------------------------------------------------------
-- Title   : FreqDivider
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Frequency dividers configuration
--
--   Switch configuration:
--   SW5 | SW4 | SW3 | SW2 | SW1 | SW0 ||
--    0  |  0  |  0  |  0  |  0  |  0  || division by 1  (3 GHz) for MAX IV ; no PLL
--    0  |  0  |  0  |  0  |  0  |  1  || division by 1  (2.8 GHz) ; with PLL
--    0  |  0  |  0  |  0  |  1  |  0  || division by 8  (352 MHz) ; TEMPO
--    0  |  0  |  0  |  0  |  1  |  1  || division by 32 (88 MHz)  ; CRISTAL
--    0  |  0  |  0  |  1  |  0  |  0  || division by 64 (44 MHz)  ; ODE
--
--   Frequency division by 64 (SOLEIL configuration):
--               BYP | DIV 1 | N0 | N1 | N2 | S0 || division ration
--   Divider 0 |  0  |   0   | 1  | 0  | 0  | 0  ||  4
--   Divider 1 |  0  |   0   | 1  | 1  | 1  | 0  ||  16
--   Divider 2 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--
--   Frequency division by 32 (SOLEIL configuration):
--               BYP | DIV 1 | N0 | N1 | N2 | S0 || division ration
--   Divider 0 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--   Divider 1 |  0  |   0   | 1  | 1  | 1  | 0  ||  16
--   Divider 2 |  0  |   1   | 0  | 0  | 0  | 0  ||  2
--
--   Frequency division by  8 (SOLEIL configuration):
--               BYP | DIV 1 | N0 | N1 | N2 | S0 || division ration
--   Divider 0 |  0  |   0   | 1  | 0  | 0  | 0  ||  4
--   Divider 1 |  1  |   0   | 0  | 0  | 1  | 0  ||  1
--   Divider 2 |  0  |   1   | 0  | 0  | 0  | 0  ||  2
--
--   Frequency division by 1 (SOLEIL configuration, with PLL):
--               BYP | DIV 1 | N0 | N1 | N2 | S0 || division ration
--   Divider 0 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--   Divider 1 |  1  |   0   | 0  | 0  | 1  | 0  ||  1
--   Divider 2 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--
--   Frequency division by 1 (MAX IV configuration, no PLL):
--               BYP | DIV 1 | N0 | N1 | N2 | S0 || division ration
--   Divider 0 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--   Divider 1 |  1  |   0   | 0  | 0  | 1  | 0  ||  1
--   Divider 2 |  1  |   0   | 0  | 0  | 0  | 0  ||  1
--------------------------------------------------------------------------------
-- File     : FreqDivider_top.vhd
-- Revision : x.x.x
-- Created  : August 8, 2016
-- Updated  : March 06, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity FreqDivider_top is
  port (
    p_reset    : in  std_logic; -- reset input
    p_SW       : in  std_logic_vector (5 downto 0); -- SOLEIL / MAX IV configuration
    p_FD0_N0   : out std_logic; -- frequency divider
    p_FD0_N1   : out std_logic; -- frequency divider
    p_FD0_BYP  : out std_logic; -- frequency divider
    p_FD1_N0   : out std_logic; -- frequency divider
    p_FD1_N1   : out std_logic; -- frequency divider
    p_FD1_BYP  : out std_logic; -- frequency divider
    p_FD2_BYP  : out std_logic; -- frequency divider
    p_FD2_DIV1 : out std_logic; -- frequency divider
    p_FD_ENA   : out std_logic; -- frequency divider buffer enable A
    p_FD_ENB   : out std_logic; -- frequency divider buffer enable B
    p_FD_ENC   : out std_logic; -- frequency divider buffer enable C
    p_FD_END   : out std_logic  -- frequency divider buffer enable D
  );
end entity FreqDivider_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_FreqDivider_top of FreqDivider_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_FD1_N0  : std_logic := '0';
  signal s_FD0_BYP : std_logic := '0';
  signal s_FD1_BYP : std_logic := '0';
  signal s_FD2_BYP : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
    -- Divider 1
    with p_SW select
      p_FD0_N0 <=
        '0' when "000001", -- bypass   ; total : div by 1
        '1' when "000010", -- div by 4 ; total : div by 8
        '0' when "000011", -- bypass   ; total : div by 32
        '1' when "000100", -- div by 4 ; total : div by 64
        '0' when OTHERS;   -- bypass   ; MAX IV, total : div by 1
    with p_SW select
      p_FD0_N1 <=
        '0' when "000001", -- bypass   ; total : div by 1
        '0' when "000010", -- div by 4 ; total : div by 8
        '0' when "000011", -- bypass   ; total : div by 32
        '0' when "000100", -- div by 4 ; total : div by 64
        '0' when OTHERS;   -- bypass   ; MAX IV, total : div by 1
    with p_SW select
      s_FD0_BYP <=
        '1' when "000001", -- bypass   ; total : div by 1
        '0' when "000010", -- div by 4 ; total : div by 8
        '1' when "000011", -- bypass   ; total : div by 32
        '0' when "000100", -- div by 4 ; total : div by 64
        '1' when OTHERS;   -- bypass   ; MAX IV, total : div by 1

    -- Divider 2
    with p_SW select
      s_FD1_N0 <=
        '0' when "000001", -- bypass    ; total : div by 1
        '0' when "000010", -- bypass    ; total : div by 8
        '1' when "000011", -- div by 16 ; total : div by 32
        '1' when "000100", -- div by 16 ; total : div by 64
        '0' when OTHERS;   -- bypass    ; MAX IV, total : div by 1
    with p_SW select
      p_FD1_N1 <=
        '0' when "000001", -- bypass    ; total : div by 1
        '1' when "000010", -- div by 16 ; total : div by 8
        '1' when "000011", -- div by 16 ; total : div by 32
        '1' when "000100", -- div by 16 ; total : div by 64
        '0' when OTHERS;   -- bypass    ; MAX IV, total : div by 1
    with p_SW select
      s_FD1_BYP <=
        '1' when "000001", -- bypass    ; total : div by 1
        '1' when "000010", -- bypass    ; total : div by 8
        '0' when "000011", -- div by 16 ; total : div by 32
        '0' when "000100", -- div by 16 ; total : div by 64
        '1' when OTHERS;   -- bypass    ; MAX IV, total : div by 1

    -- Divider 3
    with p_SW select
      s_FD2_BYP <=
        '1' when "000001", -- bypass    ; total : div by 1
        '0' when "000010", -- div by 2  ; total : div by 8
        '0' when "000011", -- div by 2  ; total : div by 32
        '1' when "000100", -- bypass    ; total : div by 64
        '1' when OTHERS;   -- bypass    ; MAX IV, total : div by 1
    with p_SW select
      p_FD2_DIV1 <=
        '0' when "000001", -- bypass    ; total : div by 1
        '1' when "000010", -- div by 2  ; total : div by 8
        '1' when "000011", -- div by 2  ; total : div by 32
        '0' when "000100", -- bypass    ; total : div by 64
        '0' when OTHERS;   -- bypass    ; MAX IV, total : div by 1

    p_FD1_N0  <= s_FD1_N0;
    p_FD0_BYP <= s_FD0_BYP;
    p_FD1_BYP <= s_FD1_BYP;
    p_FD2_BYP <= s_FD2_BYP;

    p_FD_ENA <= '1';
    p_FD_ENB <= '1';
    p_FD_ENC <= '1';
    p_FD_END <= '1';

end architecture rtl_FreqDivider_top;
