--------------------------------------------------------------------------------
-- Title   : Hermes
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : interface between the BBB and other components
--
-- CMD register => bit 0 = board reset, upon reset this bit must be set to '0' !!!
--                          0 => no reset
--                          1 => reset
--------------------------------------------------------------------------------
-- File     : Hermes_top.vhdl
-- Revision : x.x.x
-- Created  : November 04, 2016
-- Updated  : November 14, 2017
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2017 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.Hermes_registers.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity Hermes_top is
  port (
    p_clkin : in std_logic; -- FPGA clock
    p_reset : in std_logic; -- reset input

    -- From the BBB
    p_address       : in std_logic_vector (6 downto 0);  -- components adress
    p_data_from_BBB : in std_logic_vector (23 downto 0); -- data from the BBB
    p_read_ready    : in std_logic; -- data from the BBB ready to be read
    p_write_request : in std_logic; -- the BBB request a data

    -- To the BBB
    p_data_to_BBB      : out std_logic_vector (7 downto 0); -- data to the BBB
    p_write_data_ready : out std_logic; -- data ready to be sent to the BBB
                                        -- OR'ed between all bits of s_data_ready

    -- Read registers (data from BBB to components)
    p_command_reg          : out std_logic_vector (23 downto 0); -- command register
    p_DAC_reg              : out std_logic_vector (31 downto 0); -- set I, Q register
    p_Temperatue_setup_reg : out std_logic_vector (23 downto 0); -- set temperature register

    -- Write registers (data from components to BBB)
    p_ADC_datas      : in std_logic_vector (33 downto 0); -- ADC convertion results
    p_Temperatue_reg : in std_logic_vector (31 downto 0);

    p_latch_data   : out std_logic_vector (7 downto 0); -- latch the data from the BBB and
                                                        -- send to a component
    p_request_data : out std_logic_vector (7 downto 0); -- request a data from a component
    p_data_ready   : in  std_logic_vector (7 downto 0); -- data from a component to send to
                                                        -- the BBB

    p_test : out std_logic_vector (1 downto 0)
  );
end entity Hermes_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_Hermes_top of Hermes_top is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_read_error     : std_logic_vector (1 downto 0) := (OTHERS => '0'); -- address error
  signal s_write_error    : std_logic_vector (1 downto 0) := (OTHERS => '0'); -- address error
  signal s_data_ready     : std_logic := '0';
  signal s_ADC_CH0        : std_logic_vector (33 downto 0) := (OTHERS => '0'); -- save the ADC CH0 values
  signal s_ADC_CH1        : std_logic_vector (33 downto 0) := (OTHERS => '0'); -- save the ADC CH1 values
  signal s_internal_ready : std_logic := '0';

  signal s_test : std_logic_vector (1 downto 0) := (OTHERS => '0');
  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Read access
  -- Read data from BBB and write it to internal registers
  ------------------------------------------------------------------------------
  process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if (p_read_ready = '1') then
        case p_address is
          when c_add_command =>
            p_command_reg   <= p_data_from_BBB;
            p_latch_data    <= "00000001";
            s_read_error(1) <= '0';
          when c_add_I =>
            p_DAC_reg       <= "01110011" & p_data_from_BBB(17 downto 0) & "000000";
            p_latch_data    <= "00000010";
            s_read_error(1) <= '0';
          when c_add_Q =>
            p_DAC_reg       <= "01110000" & p_data_from_BBB(17 downto 0) & "000000";
            p_latch_data    <= "00000010";
            s_read_error(1) <= '0';
          when c_add_Temperatue_setup =>
            p_Temperatue_setup_reg <= p_data_from_BBB;
            p_latch_data           <= "00000100";
            s_read_error(1)        <= '0';
          when others =>
            p_latch_data    <= (OTHERS => '0');
            s_read_error(1) <= '1';
        end case;
      else
        p_latch_data    <= (OTHERS => '0');
        s_read_error(1) <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  --**************************************************************************--
  --**************************************************************************--
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Write access
  -- Write data to the BBB from internal registers
  ------------------------------------------------------------------------------

  -- Request data
  process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if (p_write_request = '1') then
        case p_address is
          when c_add_Vc_0 =>
            -- ADC CH0
            p_request_data   <= "00000001";
            s_internal_ready <= '0';
          when c_add_Vc_1 =>
            -- ADC CH0
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Vc_2 =>
            -- ADC CH0
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Vs_0 =>
            -- ADC CH1
            p_request_data   <= "00000010";
            s_internal_ready <= '0';
          when c_add_Vs_1 =>
            -- ADC CH1
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Vs_2 =>
            -- ADC CH1
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Temp_0 =>
            -- Temperature controler
            p_request_data <= "00000011";
            s_internal_ready <= '0';
          when c_add_Temp_1 =>
            -- Temperature controler
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Temp_2 =>
            -- Temperature controler
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when c_add_Temp_3 =>
            -- Temperature controler
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '1';
          when others =>
            p_request_data   <= (OTHERS => '0');
            s_internal_ready <= '0';
        end case;
      else
        p_request_data   <= (OTHERS => '0');
        s_internal_ready <= '0';
      end if;
    end if;
  end process;

  -- Latch data when ready to send to the BBB
  process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if ((s_data_ready = '1') or (s_internal_ready = '1')) then
        case p_address is
          when c_add_Vs_0 =>
            s_ADC_CH0        <= p_ADC_datas;
            p_data_to_BBB    <= p_ADC_datas(7 downto 0);
            s_write_error(1) <= '0';
            s_test(0)  <= '1';
          when c_add_Vs_1 =>
            p_data_to_BBB    <= s_ADC_CH0(15 downto 8);
            s_write_error(1) <= '0';
          when c_add_Vs_2 =>
            p_data_to_BBB    <= s_ADC_CH0(23 downto 16);
            s_write_error(1) <= '0';
          when c_add_Vc_0 =>
            s_ADC_CH1        <= p_ADC_datas;
            p_data_to_BBB    <= p_ADC_datas(7 downto 0);
            s_write_error(1) <= '0';
            s_test(1)  <= '1';
          when c_add_Vc_1 =>
            p_data_to_BBB    <= s_ADC_CH1(15 downto 8);
            s_write_error(1) <= '0';
          when c_add_Vc_2 =>
            p_data_to_BBB    <= s_ADC_CH1(23 downto 16);
            s_write_error(1) <= '0';
          when c_add_Temp_0 =>
            p_data_to_BBB    <= p_Temperatue_reg(7 downto 0);
            s_write_error(1) <= '0';
          when c_add_Temp_1 =>
            p_data_to_BBB    <= p_Temperatue_reg(15 downto 8);
            s_write_error(1) <= '0';
          when c_add_Temp_2 =>
            p_data_to_BBB    <= p_Temperatue_reg(23 downto 16);
            s_write_error(1) <= '0';
          when c_add_Temp_3 =>
            p_data_to_BBB    <= p_Temperatue_reg(31 downto 24);
            s_write_error(1) <= '0';
          when others =>
            s_write_error(1) <= '1';
        end case;
      else
        s_write_error(1) <= '0';
        s_test(0)  <= '0';
        s_test(1)  <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_data_ready       <= or_reduce(p_data_ready);
  p_write_data_ready <= s_data_ready or s_internal_ready;

  p_test(0)  <= s_test(0);
  p_test(1)  <= s_test(1);

end architecture rtl_Hermes_top;
