--------------------------------------------------------------------------------
-- Title   : PLL configuration
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : configuration of the PLL through the SPI port
--------------------------------------------------------------------------------
-- File     : PLL_configuration.vhd
-- Revision : x.x.x
-- Created  : August 12, 2016
-- Updated  : March 06, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.PLL_lib.PLL_register_array;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity PLL_configuration is
  generic (
    g_registers       : PLL_register_array;
    g_nb_of_registers : integer;
    g_registers_size  : integer
  );
  port (
    p_clkin         : in  std_logic;
    p_reset         : in  std_logic; -- reset input
    p_laser_trigger : in  std_logic;
    p_PLL_sync      : out std_logic; -- PLL synchronization with CLK-SR
    p_PLL_LE        : out std_logic; -- PLL LE
    p_PLL_CLK       : out std_logic; -- PLL clock
    p_PLL_DATA      : out std_logic; -- PLL data
    p_PLL_ENA       : out std_logic; -- PLL buffer enable A
    p_PLL_ENB       : out std_logic; -- PLL buffer enable B
    p_PLL_ENC       : out std_logic; -- PLL buffer enable C
    p_PLL_END       : out std_logic  -- PLL buffer enable D
  );
end entity PLL_configuration;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_PLL_configuration of PLL_configuration is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_start_configuration    : std_logic := '0';
  signal s_finished_configuration : std_logic := '0';
  signal s_registers_index        : integer   := 0;
  signal s_SPI_cnt                : unsigned (7 downto 0) := (others => '0'); -- state machine counter
  signal s_data                   : std_logic_vector (g_registers_size downto 0) := (others => '0');
  signal s_push_data_out          : std_logic := '0';

  -- PLL chip signals
  signal s_PLL_LE   : std_logic := '0';
  signal s_PLL_sync : std_logic := '1';
  signal s_PLL_CLK  : std_logic := '0';
  signal s_PLL_DATA : std_logic := '0';
  signal s_PLL_ENA  : std_logic := '0';
  signal s_PLL_ENB  : std_logic := '0';
  signal s_PLL_ENC  : std_logic := '0';
  signal s_PLL_END  : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Start the configuration of the PLL
  -- The configuration start by seting the p_reset
  -- It stops when all registers has been pushed out
  -- Buffers between the digital board and the RF board are set and unset
  -- during the transmission
  start_configuration : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if (p_reset = '1') then
        -- Start the process of pushing the registers to the output
        s_start_configuration <= '1';
        -- Set the buffer signals
        s_PLL_ENA <= '1';
        s_PLL_ENB <= '1';
        s_PLL_ENC <= '1';
        s_PLL_END <= '1';
      elsif (s_start_configuration = '1') then
        if (s_finished_configuration = '1') then
          -- All registers has been pushed out
          -- Stop and wait for next commande through p_reset signal
          s_start_configuration <= '0'; -- stay here forever until a reset
          -- Unset the buffer signals
          s_PLL_ENA <= '0';
          s_PLL_ENB <= '0';
          s_PLL_ENC <= '0';
          s_PLL_END <= '0';
        end if;
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- SPI state machine
  -- Push each bit of a register to the output then fetch the next register
  -- The current register is selected with s_registers_index
  SPI_manager : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if (s_start_configuration = '1') then
        case s_SPI_cnt is
          when to_unsigned(0, s_SPI_cnt'HIGH + 1) =>
            -- Start to push each bit of the current
            -- register to the output
            s_SPI_cnt       <= s_SPI_cnt + 1;
            s_push_data_out <= '1';
          when to_unsigned((g_registers_size * 2) + 3, s_SPI_cnt'HIGH + 1) =>
            -- Stop to push each bit out
            s_SPI_cnt       <= s_SPI_cnt + 1;
            s_push_data_out <= '0';
          when to_unsigned((g_registers_size * 2) + 4, s_SPI_cnt'HIGH + 1) =>
            -- Latch data to PLL
            s_SPI_cnt <= s_SPI_cnt + 1;
            s_PLL_LE  <= '1';
          when to_unsigned((g_registers_size * 2) + 5, s_SPI_cnt'HIGH + 1) =>
            -- Release latch
            s_SPI_cnt <= s_SPI_cnt + 1;
            s_PLL_LE  <= '0';
          when to_unsigned((g_registers_size * 2) + 6, s_SPI_cnt'HIGH + 1) =>
            -- End of transmission of the current register
            s_SPI_cnt      <= (others => '0');
            s_push_data_out <= '0';
            if (s_registers_index = g_nb_of_registers) then
              -- All registers has been pushed out
              s_finished_configuration <= '1';
            else
              -- Switch to the next register
              s_registers_index <= s_registers_index + 1;
            end if;
            s_PLL_LE <= '0';
          when others =>
            s_SPI_cnt <= s_SPI_cnt + 1;
        end case;
      else
        s_SPI_cnt       <= (others => '0');
        s_push_data_out <= '0';
        -- Initialization of the index to the first register
        s_registers_index <= 0;
        s_finished_configuration <= '0';
        s_PLL_LE <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Shift register to push each bit of the current register to the output
  bit_transmission : process (p_clkin)
  begin
    if (falling_edge(p_clkin)) then
      if (s_push_data_out = '1') then
        case s_SPI_cnt(0) is
          when '1' =>
            -- Push out one bit
            s_PLL_DATA <= s_data(g_registers_size);
            s_data (g_registers_size downto 1) <= s_data ((g_registers_size - 1) downto 0);
            s_PLL_CLK <= '0';
          when others =>
            s_PLL_CLK <= '1';
        end case;
      else
        -- Idle state
        s_PLL_DATA <= '0';
        s_data     <= g_registers(s_registers_index);
        s_PLL_CLK  <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Synchronize of the PLL to the laser trigger
  synchronise_to_trigger : entity work.PLL_synchronize (rtl_PLL_synchronize)
    port map (
      p_clkin         => p_clkin,
      p_reset         => p_reset,
      p_start_sync    => s_finished_configuration,
      p_laser_trigger => p_laser_trigger,
      p_PLL_sync      => s_PLL_sync
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_PLL_LE   <= s_PLL_LE;   -- PLL LE
  p_PLL_CLK  <= s_PLL_CLK;  -- PLL clock
  p_PLL_DATA <= s_PLL_DATA; -- PLL data
  p_PLL_sync <= s_PLL_sync; -- PLL synchronization with CLK-SR
  p_PLL_ENA  <= s_PLL_ENA;  -- PLL buffer enable A
  p_PLL_ENB  <= s_PLL_ENB;  -- PLL buffer enable B
  p_PLL_ENC  <= s_PLL_ENC;  -- PLL buffer enable C
  p_PLL_END  <= s_PLL_END;  -- PLL buffer enable D

end architecture rtl_PLL_configuration;
