--------------------------------------------------------------------------------
-- Title   : synchronize to a externall trigger
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : synchronize of the PLL to the laser trigger.
--               The output of the PLL is disabled efter the configuration of
--               registers. It is enabled again on the rising edge of the laser
--               trigger.
--------------------------------------------------------------------------------
-- File     : PLL_synchronize.vhd
-- Revision : x.x.x
-- Created  : February 21, 2018
-- Updated  : March 21, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity PLL_synchronize is
  port (
    p_clkin         : in  std_logic; -- 20 MHz
    p_reset         : in  std_logic; -- reset input
    p_start_sync    : in  std_logic;
    p_laser_trigger : in  std_logic; -- laser trigger signal
    p_PLL_sync      : out std_logic  -- PLL synchronization with CLK-SR
  );
end entity PLL_synchronize;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_PLL_synchronize of PLL_synchronize is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_cntmax : unsigned (31 downto 0) := X"00000006"; -- #3 MHz

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_square_trigger             : std_logic := '0';
  signal s_rising_edge_laser_trigger  : std_logic := '0';
  signal s_falling_edge_laser_trigger : std_logic := '0';
  signal s_laser_trigger              : std_logic := '0';

  signal s_PLL_sync            : std_logic := '1';
  signal s_start_sync          : std_logic := '0';
  signal s_stop_sync           : std_logic := '0';
  signal s_wait_for_next_trig  : std_logic := '0';
  signal s_wait_for_async_trig : std_logic := '1';
  signal s_cnt                 : unsigned (31 downto 0) := (OTHERS => '0'); -- counter

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- The storage ring clock is a 10ns pulse at SOLEIL. If it is used as laser
  -- trigger instead of the laser 1KHz, the pulse width is too short to be detected.
  -- This process make a square wave of the pulse. The square wave's width should
  -- be at least 0.1 s.
  square_laser_trigger : process (p_laser_trigger)
  begin
    if (rising_edge(p_laser_trigger)) then
      s_square_trigger <= not s_square_trigger;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Debounce the laser trigger clock
  debounce_laser_trigger : entity work.PLL_debounce (rtl_PLL_debounce)
    generic map (
      g_size  => 3 )
    port map (
      p_clkin        => p_clkin,
      p_in           => s_square_trigger,
      p_rising_edge  => s_rising_edge_laser_trigger,
      p_falling_edge => s_falling_edge_laser_trigger,
      p_out          => s_laser_trigger
    );

  ------------------------------------------------------------------------------
  -- At the end of the configuration of the registers a synchronization to laser
  -- trigger is asked. p_start_sync goes up for one p_clkin. this process latch
  -- the request and disable the PLL's output.
  start_synchronize : process (s_stop_sync, p_clkin)
  begin
    if (s_stop_sync = '1') then
        s_start_sync <= '0';
        s_PLL_sync  <= '1';
    elsif (falling_edge(p_clkin)) then
      if (p_start_sync = '1') then
        s_start_sync <= '1'; -- stay active until the end of the synchronization
        s_PLL_sync  <= '0'; -- disables the PLL's output
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Wait for the next (resynchronized) laser trigger
  wait_trigger : process (s_stop_sync, p_clkin)
  begin
    if (s_stop_sync = '1') then
      s_wait_for_next_trig <= '0';
    elsif (falling_edge(p_clkin)) then
      if ((s_start_sync = '1') and (s_rising_edge_laser_trigger = '1')) then
        s_wait_for_next_trig <= '1';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Keep the output of the PLL disabled during 300 s at least.
  -- Also avoid metastability as long as the laser trigger's frequency is
  -- above 3 MHz (laser trigger is #1 KHz ; CLK_ANS is 846.7 kHz).
  count_down : process (s_stop_sync, p_clkin)
  begin
    if (s_stop_sync = '1') then
      s_cnt <= (OTHERS => '0');
      s_wait_for_async_trig <= '1';
    elsif (falling_edge(p_clkin)) then
      if (s_wait_for_next_trig = '1') then
        if (s_cnt = c_cntmax) then
          s_cnt <= s_cnt;
          s_wait_for_async_trig <= '0';
        else
          s_cnt <= s_cnt + 1;
          s_wait_for_async_trig <= '1';
        end if;
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Wait for the next (asynchrone) laser trigger
  wait_async_trigger : process (s_wait_for_async_trig, s_square_trigger)
  begin
    if (s_wait_for_async_trig = '1') then
      s_stop_sync <= '0';
    elsif (rising_edge(s_square_trigger)) then
      s_stop_sync <= '1';
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_PLL_sync <= not s_PLL_sync;

end architecture rtl_PLL_synchronize;
