--------------------------------------------------------------------------------
-- Title   : PLL
-- Project : TimIQ
--------------------------------------------------------------------------------
-- Description : Configure the PLL for frequency multiplication by 8
--  PLL is inhibited in MAX IV configuration. the configuration (SOLEIL / MAX IV)
--  is done with the switch on the board:
--  SW 5-4-3-2-1-0
--     0-0-0-0-0-1 : SOLEIL
--     0-0-0-0-0-0 : MAX IV
--------------------------------------------------------------------------------
-- File     : PLL_top.vhd
-- Revision : x.x.x
-- Created  : August 08, 2016
-- Updated  : March 21, 2018
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr - https://www.maxiv.lu.se
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2015 - 2018 Synchrotron Soleil - MAX IV Laboratory
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.PLL_lib.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity PLL_top is
  port (
    p_reset     : in  std_logic; -- reset input
    p_clkin     : in  std_logic; -- CLK input
    p_cmd_REG1  : in  std_logic; -- command register bit 0 ; p_CSW = "000000000000001"
    p_CSW       : in  std_logic; -- chip select of internal write transfert
    p_SW        : in  std_logic_vector (5 downto 0); -- SOLEIL / MAX IV configuration
    p_PLL_LE    : out std_logic; -- PLL LE
    p_PLL_CE    : out std_logic; -- PLL CE
    p_PLL_CLK   : out std_logic; -- PLL clock
    p_PLL_DATA  : out std_logic; -- PLL data
    p_PLL_ENout : out std_logic; -- PLL ENout
    p_PLL_ENA   : out std_logic; -- PLL buffer enable A
    p_PLL_ENB   : out std_logic; -- PLL buffer enable B
    p_PLL_ENC   : out std_logic; -- PLL buffer enable C
    p_PLL_END   : out std_logic; -- PLL buffer enable D

    p_PLL_sync      : out std_logic; -- PLL synchronization with CLK-SR
    p_laser_trigger : in std_logic   -- laser trigger clock
  );
end entity PLL_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_PLL_top of PLL_top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_recalibrate       : std_logic := '0';
  signal s_start_recalibrate : std_logic := '0';
  signal s_reset_recalibrate : std_logic := '0';

  -- PLL chip signals
  signal s_PLL_sync_config : std_logic := '1';
  signal s_PLL_sync_calib  : std_logic := '1';
  signal s_PLL_LE_config   : std_logic := '0';
  signal s_PLL_CLK_config  : std_logic := '0';
  signal s_PLL_DATA_config : std_logic := '0';
  signal s_PLL_ENA_config  : std_logic := '0';
  signal s_PLL_ENB_config  : std_logic := '0';
  signal s_PLL_ENC_config  : std_logic := '0';
  signal s_PLL_END_config  : std_logic := '0';
  signal s_PLL_LE_calib    : std_logic := '0';
  signal s_PLL_CLK_calib   : std_logic := '0';
  signal s_PLL_DATA_calib  : std_logic := '0';
  signal s_PLL_ENA_calib   : std_logic := '0';
  signal s_PLL_ENB_calib   : std_logic := '0';
  signal s_PLL_ENC_calib   : std_logic := '0';
  signal s_PLL_END_calib   : std_logic := '0';

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  ------------------------------------------------------------------------------
  -- Configuration of the PLL trigerred by the p_reset signal
  configure_PLL : entity work.PLL_configuration (rtl_PLL_configuration)
    generic map (
      g_registers       => c_PLL_reg_configuration,
      g_nb_of_registers => c_PLL_reg_configuration'HIGH,
      g_registers_size  => c_PLL_reg_configuration(0)'HIGH )
    port map (
      p_clkin         => p_clkin,
      p_reset         => p_reset,
      p_laser_trigger => p_laser_trigger,
      p_PLL_sync      => s_PLL_sync_config,
      p_PLL_LE        => s_PLL_LE_config,
      p_PLL_CLK       => s_PLL_CLK_config,
      p_PLL_DATA      => s_PLL_DATA_config,
      p_PLL_ENA       => s_PLL_ENA_config,
      p_PLL_ENB       => s_PLL_ENB_config,
      p_PLL_ENC       => s_PLL_ENC_config,
      p_PLL_END       => s_PLL_END_config
    );

  ------------------------------------------------------------------------------
  -- Calibration of the PLL's VCO trigerred by the p_CSW and p_cmd_REG1 signal
  PLL_calibration_command : process (p_clkin)
  begin
    if (rising_edge(p_clkin)) then
      if ((p_CSW = '1') and (p_cmd_REG1 = '1')) then
        s_recalibrate <= '1'; -- recalibrate the PLL
      else
        s_recalibrate <= '0';
      end if;
    end if;
  end process;

  calibartion_PLL : entity work.PLL_calibration (rtl_PLL_calibration)
    generic map (
      g_registers      => c_PLL_reg_calibration,
      g_registers_size => c_PLL_reg_calibration'HIGH )
    port map (
      p_clkin         => p_clkin,
      p_reset         => s_recalibrate,
      p_laser_trigger => p_laser_trigger,
      p_PLL_sync      => s_PLL_sync_calib,
      p_PLL_LE        => s_PLL_LE_calib,
      p_PLL_CLK       => s_PLL_CLK_calib,
      p_PLL_DATA      => s_PLL_DATA_calib,
      p_PLL_ENA       => s_PLL_ENA_calib,
      p_PLL_ENB       => s_PLL_ENB_calib,
      p_PLL_ENC       => s_PLL_ENC_calib,
      p_PLL_END       => s_PLL_END_calib
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  p_PLL_sync <= s_PLL_sync_config or s_PLL_sync_calib;
  p_PLL_LE   <= s_PLL_LE_config   or s_PLL_LE_calib;
  p_PLL_CLK  <= s_PLL_CLK_config  or s_PLL_CLK_calib;
  p_PLL_DATA <= s_PLL_DATA_config or s_PLL_DATA_calib;
  p_PLL_ENA  <= s_PLL_ENA_config  or s_PLL_ENA_calib;
  p_PLL_ENB  <= s_PLL_ENB_config  or s_PLL_ENB_calib;
  p_PLL_ENC  <= s_PLL_ENC_config  or s_PLL_ENC_calib;
  p_PLL_END  <= s_PLL_END_config  or s_PLL_END_calib;

  with p_SW select
    p_PLL_ENout <=
      '0' when "000000", -- MAX IV
      '1' when OTHERS;   -- SOLEIL

  with p_SW select
    p_PLL_CE <=
      '0' when "000000", -- MAX IV
      '1' when OTHERS;   -- SOLEIL

end architecture rtl_PLL_top;
